import { buildASTSchema, buildSchema, isSchema, } from 'graphql';
import { createYoga, } from 'graphql-yoga';
import { useSupergraph } from '@graphql-mesh/fusion-runtime';
import { isPromise } from '@whatwg-node/server';
export function createMeshHTTPHandler(config) {
    let handlerImportFn;
    if (config.handlers != null) {
        if (typeof config.handlers === 'function') {
            handlerImportFn = config.handlers;
        }
        else {
            handlerImportFn = (handlerName) => config.handlers[handlerName];
        }
    }
    else {
        handlerImportFn = defaultHandlerImport;
    }
    let fetchAPI = config.fetchAPI;
    // eslint-disable-next-line prefer-const
    let logger;
    function defaultHandlerImport(handlerName) {
        const omnigraphPackageName = `@omnigraph/${handlerName}`;
        logger.info(`Loading ${omnigraphPackageName}`);
        return import(omnigraphPackageName);
    }
    const yoga = createYoga({
        fetchAPI: config.fetchAPI,
        logging: config.logging,
        plugins: [
            ...(config.plugins || []),
            useSupergraph({
                getSupergraph() {
                    if (typeof config.supergraph === 'function') {
                        return config.supergraph();
                    }
                    if (isSchema(config.supergraph)) {
                        return config.supergraph;
                    }
                    if (typeof config.supergraph === 'string') {
                        let url = config.supergraph;
                        if (url.startsWith('.')) {
                            url = `file:///${url}`;
                        }
                        yoga.logger.info(`Fetching Supergraph from ${url}`);
                        return fetchAPI
                            .fetch(config.supergraph, {
                            headers: config.schemaHeaders,
                        })
                            .then(res => res.text())
                            .then(schemaString => buildSchema(schemaString, {
                            assumeValid: true,
                            assumeValidSDL: true,
                            noLocation: true,
                        }));
                    }
                    return buildASTSchema(config.supergraph, {
                        assumeValid: true,
                        assumeValidSDL: true,
                    });
                },
                getExecutorFromHandler(handlerName, handlerOptions, getSubgraph) {
                    function handleImportResult(importRes) {
                        const getSubgraphExecutor = importRes.getSubgraphExecutor;
                        if (!getSubgraphExecutor) {
                            logger.error(`getSubgraphExecutor is not exported from the handler: ${handlerName}`);
                            throw new Error(`getSubgraphExecutor is not exported from the handler: ${handlerName}`);
                        }
                        return getSubgraphExecutor({
                            getSubgraph,
                            options: handlerOptions,
                        });
                    }
                    const importRes$ = handlerImportFn(handlerName);
                    if (isPromise(importRes$)) {
                        return importRes$.then(handleImportResult);
                    }
                    return handleImportResult(importRes$);
                },
                polling: config.polling,
            }),
        ],
        cors: config.cors,
        graphiql: config.graphiql,
        batching: config.batching,
    });
    fetchAPI ||= yoga.fetchAPI;
    logger = yoga.logger;
    return yoga;
}
